package com.taais.biz.listener;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import com.alibaba.excel.context.AnalysisContext;
import com.alibaba.excel.event.AnalysisEventListener;
import com.taais.common.core.core.domain.CommonResult;
import com.taais.common.core.exception.ServiceException;
import com.taais.common.core.utils.SpringUtils;
import com.taais.common.core.utils.ValidatorUtils;
import com.taais.common.excel.core.ExcelListener;
import com.taais.common.excel.core.ExcelResult;
import com.taais.biz.domain.bo.AlgorithmModelTrackBo;
import com.taais.biz.domain.vo.AlgorithmModelTrackImportVo;
import com.taais.biz.domain.vo.AlgorithmModelTrackVo;
import com.taais.biz.service.IAlgorithmModelTrackService;
import lombok.extern.slf4j.Slf4j;

import java.util.List;

/**
 * 算法模型配置自定义导入
 *
 * @author wangkang
 */
@Slf4j
public class AlgorithmModelTrackImportListener extends AnalysisEventListener<AlgorithmModelTrackImportVo> implements ExcelListener<AlgorithmModelTrackImportVo> {
    private final IAlgorithmModelTrackService algorithmModelTrackService;

    private final Boolean isUpdateSupport;
    private int successNum = 0;
    private int failureNum = 0;
    private final StringBuilder successMsg = new StringBuilder();
    private final StringBuilder failureMsg = new StringBuilder();

    public AlgorithmModelTrackImportListener(Boolean isUpdateSupport) {
        this.algorithmModelTrackService = SpringUtils.getBean(IAlgorithmModelTrackService.class);
        this.isUpdateSupport = isUpdateSupport;
    }

    @Override
    public void invoke(AlgorithmModelTrackImportVo algorithmModelTrackVo, AnalysisContext context) {
        try {

            AlgorithmModelTrackBo algorithmModelTrackBo = BeanUtil.toBean(algorithmModelTrackVo, AlgorithmModelTrackBo.class);

            //TODO:根据某个字段，查询数据库表中是否存在记录，不存在就新增，存在就更新
            AlgorithmModelTrackVo algorithmModelTrackVo1 = null;

            //algorithmModelTrackVo1 = algorithmModelTrackService.selectBySomefield(algorithmModelTrackVo.getSomefield());
            if (ObjectUtil.isNull(algorithmModelTrackVo1)) {
                //不存在就新增
                algorithmModelTrackBo.setVersion(0);
                ValidatorUtils.validate(algorithmModelTrackBo);
                CommonResult<String> result = algorithmModelTrackService.insert(algorithmModelTrackBo);

                if (result.getCode() == CommonResult.SUCCESS) {
                    successNum++;
                    successMsg.append("<br/>").append(successNum).append("、算法模型配置 记录导入成功");
                    return;
                } else {
                    failureNum++;
                    failureMsg.append("<br/>").append(failureNum).append("、算法模型配置 记录导入失败");
                    return;
                }
            } else if (isUpdateSupport) {
                //存在就更新
                algorithmModelTrackBo.setId(algorithmModelTrackVo1.getId());//主键
                algorithmModelTrackBo.setVersion(algorithmModelTrackVo1.getVersion());
                boolean updated = algorithmModelTrackService.update(algorithmModelTrackBo);
                if (updated) {
                    successNum++;
                    successMsg.append("<br/>").append(successNum).append("、算法模型配置 记录更新成功");
                    return;
                } else {
                    failureNum++;
                    failureMsg.append("<br/>").append(failureNum).append("、算法模型配置 记录更新失败");
                    return;
                }
            }
        } catch (Exception e) {
            failureNum++;
            String msg = "<br/>" + failureNum + "、算法模型配置 记录导入失败：";
            failureMsg.append(msg).append(e.getMessage());
            log.error(msg, e);
        }
    }

    @Override
    public void doAfterAllAnalysed(AnalysisContext context) {

    }

    @Override
    public ExcelResult<AlgorithmModelTrackImportVo> getExcelResult() {
        return new ExcelResult<>() {

            @Override
            public String getAnalysis() {
                if (failureNum > 0) {
                    failureMsg.insert(0, "很抱歉，导入失败！共 " + failureNum + " 条数据没有成功导入，错误如下：");
                    throw new ServiceException(failureMsg.toString());
                } else {
                    successMsg.insert(0, "恭喜您，数据已全部导入成功！共 " + successNum + " 条，数据如下：");
                }
                return successMsg.toString();
            }

            @Override
            public List<AlgorithmModelTrackImportVo> getList() {
                return null;
            }

            @Override
            public List<String> getErrorList() {
                return null;
            }
        };
    }
}
