package com.taais.biz.listener;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import com.alibaba.excel.context.AnalysisContext;
import com.alibaba.excel.event.AnalysisEventListener;
import com.taais.common.core.exception.ServiceException;
import com.taais.common.core.utils.SpringUtils;
import com.taais.common.core.utils.ValidatorUtils;
import com.taais.common.excel.core.ExcelListener;
import com.taais.common.excel.core.ExcelResult;
import com.taais.biz.domain.bo.DataSeqBo;
import com.taais.biz.domain.vo.DataSeqImportVo;
import com.taais.biz.domain.vo.DataSeqVo;
import com.taais.biz.service.IDataSeqService;
import lombok.extern.slf4j.Slf4j;

import java.util.List;

/**
 * 数据管理自定义导入
 *
 * @author wangkang
 */
@Slf4j
public class DataSeqImportListener extends AnalysisEventListener<DataSeqImportVo> implements ExcelListener<DataSeqImportVo> {
    private final IDataSeqService dataSeqService;

    private final Boolean isUpdateSupport;
    private int successNum = 0;
    private int failureNum = 0;
    private final StringBuilder successMsg = new StringBuilder();
    private final StringBuilder failureMsg = new StringBuilder();

    public DataSeqImportListener(Boolean isUpdateSupport) {
        this.dataSeqService = SpringUtils.getBean(IDataSeqService.class);
        this.isUpdateSupport = isUpdateSupport;
    }

    @Override
    public void invoke(DataSeqImportVo dataSeqVo, AnalysisContext context) {
        try {

            DataSeqBo dataSeqBo = BeanUtil.toBean(dataSeqVo, DataSeqBo.class);

            //TODO:根据某个字段，查询数据库表中是否存在记录，不存在就新增，存在就更新
            DataSeqVo dataSeqVo1 = null;

            //dataSeqVo1 = dataSeqService.selectBySomefield(dataSeqVo.getSomefield());
            if (ObjectUtil.isNull(dataSeqVo1)) {
                //不存在就新增
                dataSeqBo.setVersion(0);
                ValidatorUtils.validate(dataSeqBo);
                boolean inserted = dataSeqService.insert(dataSeqBo);

                if (inserted) {
                    successNum++;
                    successMsg.append("<br/>").append(successNum).append("、数据管理 记录导入成功");
                    return;
                } else {
                    failureNum++;
                    failureMsg.append("<br/>").append(failureNum).append("、数据管理 记录导入失败");
                    return;
                }
            } else if (isUpdateSupport) {
                //存在就更新
                dataSeqBo.setId(dataSeqVo1.getId());//主键
                dataSeqBo.setVersion(dataSeqVo1.getVersion());
                boolean updated = dataSeqService.update(dataSeqBo);
                if (updated) {
                    successNum++;
                    successMsg.append("<br/>").append(successNum).append("、数据管理 记录更新成功");
                    return;
                } else {
                    failureNum++;
                    failureMsg.append("<br/>").append(failureNum).append("、数据管理 记录更新失败");
                    return;
                }
            }
        } catch (Exception e) {
            failureNum++;
            String msg = "<br/>" + failureNum + "、数据管理 记录导入失败：";
            failureMsg.append(msg).append(e.getMessage());
            log.error(msg, e);
        }
    }

    @Override
    public void doAfterAllAnalysed(AnalysisContext context) {

    }

    @Override
    public ExcelResult<DataSeqImportVo> getExcelResult() {
        return new ExcelResult<>() {

            @Override
            public String getAnalysis() {
                if (failureNum > 0) {
                    failureMsg.insert(0, "很抱歉，导入失败！共 " + failureNum + " 条数据没有成功导入，错误如下：");
                    throw new ServiceException(failureMsg.toString());
                } else {
                    successMsg.insert(0, "恭喜您，数据已全部导入成功！共 " + successNum + " 条，数据如下：");
                }
                return successMsg.toString();
            }

            @Override
            public List<DataSeqImportVo> getList() {
                return null;
            }

            @Override
            public List<String> getErrorList() {
                return null;
            }
        };
    }
}
